//
// Encoding functions shared by both Avi2Cvc (Video) and Bmp2Cvi (Image) converters
//

public class Cv1{

	public byte []prev;

	public void set_as_previous(byte[] pix){
		for(var i=0;i<pix.Length;i++){
			prev[i]=pix[i];
		}
	}

	// 0:Uncompressed
	public int encode_none(byte[] pix, byte[] outdata){
		for(var i=0;i<pix.Length;i++){
			outdata[i]=pix[i];
		}

		set_as_previous(pix);
		return pix.Length;
	}

	// 1: Runlength encoding
	public int encode_rle(byte[] pix, byte[] outdata){
		int pos=0;
		byte pb=pix[0];
		byte cnt=1;

		for(var i=1;i<pix.Length;i++){
			if ((pix[i]==pb) && (cnt<126)){
				cnt++;
			}else{
				outdata[pos]=cnt;
				outdata[pos+1]=pb;
				pos+=2;
			
				pb=pix[i];
				cnt=1;
			}
		}
		
		if (cnt!=0){
			outdata[pos]=cnt;
			outdata[pos+1]=pb;
			pos+=2;
		}
		
		set_as_previous(pix);
		return pos;
	}


	public int encode_rle_size(byte[] pix){
		int pos=0;
		byte pb=pix[0];
		byte cnt=1;

		for(var i=1;i<pix.Length;i++){
			if ((pix[i]==pb) && (cnt<126)){
				cnt++;
			}else{
				pos+=2;
				pb=pix[i];
				cnt=1;
			}
		}
		
		if (cnt!=0){pos+=2;}
	//	writeln("RLE Size:",pos);
		return pos;
	}

	// 2: Delta1 - RLE + Skipping same as previous frame pixels
	public int encode_delta1(byte[] pix,byte[] outdata){
		int cb,pb;
		int pos=0;
		int no=1;

		pb=pix[0];
		if (pix[0]==prev[0]){pb=-1;}

		for(var i=1;i<pix.Length;i++){
			cb=pix[i];
			if (pix[i]==prev[i]){cb=-1;}

			if (cb==pb && no<126){
				no++;
			}else{
				if (pb==-1){
					outdata[pos++]=(byte)-no;
				}else{
					outdata[pos++]=(byte)no;
					outdata[pos++]=pix[i-1];
				}

				no=1;
			}

			pb=cb;
		}

		if (no!=0){
			if (pb==-1){
				outdata[pos++]=(byte)(-no);
			}else{
				outdata[pos++]=(byte)no;
				outdata[pos++]=pix[pix.Length-1];	// Last pixel
			}
		}

		set_as_previous(pix);
		return pos;
	}


	public int encode_delta1_size(byte[] pix){
		int pos=0;
		int cb,pb,no=1;

		pb=pix[0];
		if (pix[0]==prev[0]){pb=-1;}

		for(var i=1;i<pix.Length;i++){
			cb=pix[i];
			
			if (pix[i]==prev[i]){cb=-1;}

			if (cb==pb && no<126){
				no++;
			}else{
				if (pb==-1){
					pos++;
				}else{
					pos+=2;
				}
				no=1;
			}

			pb=cb;
		}

		if (no!=0){
			if (pb==-1){
				pos++;
			}else{
				pos+=2;
			}
		}
	//writeln("Delta 1 Size:",pos);
	return pos;
	}

	//3: Delta 2 - Byte copy skipping pixel same as previous frame
	public int encode_delta2(byte[] pix,byte[] outdata){
		int pos=0,no=0,xo=0;

		for(var i=0;i<pix.Length;i++){
			if (pix[i]==prev[i]){
				if (xo>0){
					outdata[pos++]=(byte)xo;
					for(int c=0;c<xo;c++){
						outdata[pos++]=pix[i-xo+c];
					}
					xo=0;
				}

				no++;
				if (no>126){
					outdata[pos++]=(byte)-no;
					no=0;
				}
			}else{
				xo++;
				if (no>0){
					outdata[pos++]=(byte)-no;
					no=0;
				}

				if (xo>=126){
					outdata[pos++]=(byte)xo;
					for(int c=0;c<xo;c++){
						outdata[pos++]=pix[i-xo+c+1];
					}
					xo=0;
				}
			}
		}

		//
		if (no>0){
			outdata[pos++]=(byte)-no;
		}

		if (xo>0){
			outdata[pos++]=(byte)xo;
			for(int c=0;c<xo;c++){
				// Should this be a 'datasize-1' ???? Bmp2Cvi test says no..
				outdata[pos++]=pix[pix.Length-xo+c];
			}
		}

		set_as_previous(pix);
		return pos;
	}


	public int encode_delta2_size(byte[] pix){
		int pos=0;
		int no=0,xo=0;

		for(var i=0;i<pix.Length;i++){
			if (pix[i]==prev[i]){
				if (xo>0){
					pos+=(1+xo);
					xo=0;
				}

				no++;

				if (no>126){
					pos++;
					no=0;
				}
			}else{
				xo++;

				if (no>0){
					pos++;
					no=0;
				}

				if (xo>=126){
					pos+=(1+xo);
					xo=0;
				}
			}
		}

		//
		if (no>0){pos++;}
		if (xo>0){pos+=(1+xo);}

		//writeln("Delta 2 Size:",pos);

		return pos;
	}
}
